#!/usr/bin/env python3
# SPDX-FileCopyrightText: 2024 Volker Krause <vkrause@kde.org>
# SPDX-License-Identifier: LGPL-2.0-or-later

import argparse
import re
import requests

parser = argparse.ArgumentParser(description='DB online ticket API response output')
parser.add_argument('--name', type=str, required=True, help='Case-sensitive last name, as on the ticket')
parser.add_argument('--ref', type=str, required=True, help='12 digit Auftragsnummer')
arguments = parser.parse_args()

postData = f'<rqfindorder version="1.0"><rqheader v="23080000" os="KCI" app="NAVIGATOR"/><rqorder on="{arguments.ref}"/><authname tln="{arguments.name}"/></rqfindorder>'
print(f"Request 1:\n{postData}\n")
req = requests.post('https://fahrkarten.bahn.de/mobile/dbc/xs.go?', data=postData.encode())
print(f"Reply 1:\n{req.text}\n")

for kwid in re.findall(r'kwid="([^"]*)"', req.text):
    postData = f'<rqorderdetails version="1.0"><rqheader v="23040000" os="KCI" app="KCI-Webservice"/><rqorder on="{arguments.ref}" kwid="{kwid}"/><authname tln="{arguments.name}"/></rqorderdetails>'
    print(f"Request 2:\n{postData}\n")
    req = requests.post('https://fahrkarten.bahn.de/mobile/dbc/xs.go?', data=postData.encode())
    print(f"Reply 2:\n{req.text}")



#!/usr/bin/env python3
# SPDX-License-Identifier: LGPL-2.0-or-later
# SPDX-FileCopyrightText: 2023 Volker Krause <vkrause@kde.org>

from itinerarytestcase import ItineraryTestCase

import unittest
from appium import webdriver
from appium.options.common.base import AppiumOptions
from appium.webdriver.common.appiumby import AppiumBy
import selenium.common.exceptions
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import WebDriverWait

import os

class ReservationTest(ItineraryTestCase):
    def setUp(self):
        # open the timeline page and import test scenario
        self.driver.find_element(by=AppiumBy.NAME, value="Itinerary").click()

        self.driver.find_element(by=AppiumBy.NAME, value="From file").click()

        self.openFile("../tests/randa2017.json")
        self.driver.find_element(by=AppiumBy.NAME, value="Import selection").click()
        self.driver.find_element(by=AppiumBy.NAME, value="MyTrip").click()

    def tearDown(self):
        self.driver.find_element(by=AppiumBy.NAME, value="Delete trip").click()
        deleteDialog = self.driver.find_element(by=AppiumBy.NAME, value="Delete Trip")
        self.assertTrue(deleteDialog.is_displayed())
        deleteDialog.find_element(by=AppiumBy.NAME, value="Delete").click()

    def test_flight(self):
        self.driver.find_element(by=AppiumBy.NAME, value="LX 963 TXL → ZRH").click()
        self.assertFalse(self.driver.find_element(by=AppiumBy.NAME, value="Barcode Scan Mode").is_displayed())
        self.assertTrue(self.driver.find_element(by=AppiumBy.NAME, value="No documents attached to this reservation.").is_enabled())
        self.driver.find_element(by=AppiumBy.NAME, value="Add Document…").click()
        self.openFile("data/iata-bcbp-demo.pdf")
        self.assertTrue(self.driver.find_element(by=AppiumBy.NAME, value="iata-bcbp-demo.pdf").is_enabled())
        # TODO document context actions not accessible
        self.driver.find_element(by=AppiumBy.NAME, value="Edit").click()
        self.goBack()
        self.goBack()

    def test_train(self):
        self.driver.find_element(by=AppiumBy.NAME, value="IC 816").click()
        self.assertTrue(self.driver.find_element(by=AppiumBy.NAME, value="Barcode Scan Mode").is_enabled())
        # TODO how to check those *don't* exist?
        # self.assertFalse(self.driver.find_element(by=AppiumBy.NAME, value="Departure Vehicle Layout").is_enabled())
        # self.assertFalse(self.driver.find_element(by=AppiumBy.NAME, value="Arrival Vehicle Layout").is_enabled())
        # self.assertTrue(self.driver.find_element(by=AppiumBy.NAME, value="Journey Details").is_enabled())
        # TODO those two aren't found for some reason??
        # self.assertFalse(self.driver.find_element(by=AppiumBy.NAME, value="Add transfer before").is_enabled())
        # self.assertFalse(self.driver.find_element(by=AppiumBy.NAME, value="Add transfer after").is_enabled())
        self.assertTrue(self.driver.find_element(by=AppiumBy.NAME, value="Alternatives").is_enabled())
        self.driver.find_element(by=AppiumBy.NAME, value="Edit").click()
        self.driver.find_element(by=AppiumBy.NAME, value="Save").click()
        self.goBack()
        # TODO we need to scroll down for this, how to do that?
        # self.driver.find_element(by=AppiumBy.NAME, value="R 241").click()
        # self.assertEqual(len(self.driver.find_elements(by=AppiumBy.NAME, value="View Indoor Map")), 2)
        # self.goBack()

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# SPDX-License-Identifier: LGPL-2.0-or-later
# SPDX-FileCopyrightText: 2023 Volker Krause <vkrause@kde.org>

from itinerarytestcase import ItineraryTestCase

import unittest
import time
from appium import webdriver
from appium.options.common.base import AppiumOptions
from appium.webdriver.common.appiumby import AppiumBy
import selenium.common.exceptions
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import WebDriverWait

import os

class TripGroupTest(ItineraryTestCase):
    def setUp(self):
        self.driver.find_element(by=AppiumBy.NAME, value="Itinerary").click()

    def testEmptyGroup(self):
        self.createAndEnterTrip("My New Trip")

        self.driver.find_element(by=AppiumBy.NAME, value="Add event").click()
        # TODO how can we check the A11y check state?
        # self.assertTrue(self.driver.find_element(by=AppiumBy.NAME, value="Add to Trip").is_checked())
        self.assertFalse(self.driver.find_element(by=AppiumBy.NAME, value="Save").is_enabled())

        self.driver.find_element(by=AppiumBy.NAME, value="Name").send_keys("Akademy 2024")
        saveBtn = self.driver.find_element(by=AppiumBy.NAME, value="Save")
        self.assertTrue(saveBtn.is_enabled())
        saveBtn.click()

        # HACK model doesn't update correctly
        # self.goBack()
        # self.driver.find_element(by=AppiumBy.NAME, value="My New Trip").click()

        self.deleteTrip("Akademy 2024")

    def testImportedGroup(self):
        self.driver.find_element(by=AppiumBy.NAME, value="MyData").click()
        self.driver.find_element(by=AppiumBy.NAME, value="Import").click()

        self.openFile("../tests/randa2017.json")
        self.driver.find_element(by=AppiumBy.NAME, value="Import selection").click()
        self.driver.find_element(by=AppiumBy.NAME, value="Randa (September 2017)").click()

        self.driver.find_element(by=AppiumBy.NAME, value="Delete trip").click()
        self.driver.find_element(by=AppiumBy.NAME, value="Delete").click()

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# SPDX-License-Identifier: LGPL-2.0-or-later
# SPDX-FileCopyrightText: 2023 Volker Krause <vkrause@kde.org>

import unittest
from appium import webdriver
from appium.options.common.base import AppiumOptions
from appium.webdriver.common.appiumby import AppiumBy
from appium.webdriver.webdriver import ExtensionBase
import selenium.common.exceptions
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import WebDriverWait

import os


class SetValueCommand(ExtensionBase):

    def method_name(self):
        return 'set_value'

    def set_value(self, element, value: str):
        """
        Set the value on this element in the application
        Args:
            value: The value to be set
        """
        data = {
            'id': element.id,
            'text': value,
        }
        return self.execute(data)['value']

    def add_command(self):
        return 'post', '/session/$sessionId/appium/element/$id/value'


# common base class for Appium tests
class ItineraryTestCase(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        options = AppiumOptions()
        options.set_capability("app", "org.kde.itinerary.desktop")
        self.driver = webdriver.Remote(command_executor='http://127.0.0.1:4723', options=options, extensions=[SetValueCommand])
        self.driver.implicitly_wait = 10

        # skip the welcome screen
        welcomeBtn = self.driver.find_element(by=AppiumBy.NAME, value="Got it!")
        welcomeBtn.click()

    @classmethod
    def tearDownClass(self):
        self.driver.quit()

    # helper methods
    def goBack(self):
        self.driver.find_element(by=AppiumBy.NAME, value="Navigate Back").click()

    def openFile(self, fileName):
        actions = ActionChains(self.driver)
        actions.send_keys(os.path.join(os.path.dirname(__file__), fileName)).perform()
        actions.send_keys(Keys.RETURN).perform()

    def openGlobalDrawer(self):
        self.driver.find_element(by=AppiumBy.NAME, value="Open drawer").click()

    def openContextDrawer(self):
        self.driver.find_elements(by=AppiumBy.NAME, value="Open drawer")[1].click()

    def enableDevMode(self):
        self.openGlobalDrawer()
        self.driver.find_element(by=AppiumBy.NAME, value="Development").click()
        self.driver.find_element(by=AppiumBy.NAME, value="Disable Development Mode").click()

    def disableDevMode(self):
        self.openGlobalDrawer()
        self.driver.find_element(by=AppiumBy.NAME, value="Development").click()
        self.driver.find_element(by=AppiumBy.NAME, value="Disable Development Mode").click()

    def createAndEnterTrip(self, name):
        self.driver.find_element(by=AppiumBy.NAME, value="Itinerary").click()

        self.driver.find_element(by=AppiumBy.NAME, value="Add trip").click()
        addDialog = self.driver.find_element(by=AppiumBy.NAME, value="Add Trip")
        self.assertTrue(addDialog.is_displayed())
        self.driver.find_element(by=AppiumBy.NAME, value="Cancel").click()

        self.driver.find_element(by=AppiumBy.NAME, value="Add trip").click()
        addDialog = self.driver.find_element(by=AppiumBy.NAME, value="Add Trip")
        saveBtn = addDialog.find_element(by=AppiumBy.NAME, value="Save")
        self.assertFalse(saveBtn.is_enabled())
        self.driver.find_element(by=AppiumBy.NAME, value="Trip name:").send_keys(name)
        self.assertTrue(saveBtn.is_enabled())
        saveBtn.click()

        self.driver.find_element(by=AppiumBy.NAME, value=name).click()

    def deleteTrip(self, name):
        self.driver.find_element(by=AppiumBy.NAME, value=name).click()
        self.goBack()

        self.driver.find_element(by=AppiumBy.NAME, value="Delete trip").click()
        self.driver.find_element(by=AppiumBy.NAME, value="Delete").click()




#!/usr/bin/env python3
# SPDX-License-Identifier: LGPL-2.0-or-later
# SPDX-FileCopyrightText: 2023 Volker Krause <vkrause@kde.org>

from itinerarytestcase import ItineraryTestCase

import unittest
from appium import webdriver
from appium.webdriver.common.appiumby import AppiumBy

import os

# Online import form tests
class ItineraryTest(ItineraryTestCase):
    # FIXME currently disabled as the DB API endpoint is no longer responding
    # def test_db(self):
    #     self.createAndEnterTrip("My New Trip")
    #
    #     action = self.driver.find_element(by=AppiumBy.NAME, value="From Deutsche Bahn reservation number")
    #     self.assertTrue(action.is_enabled())
    #     action.click()
    #
    #     searchBtn = self.driver.find_element(by=AppiumBy.NAME, value="Search…")
    #     self.assertFalse(searchBtn.is_enabled())
    #     nameInput = self.driver.find_element(by=AppiumBy.NAME, value="Family name")
    #     nameInput.send_keys("Konqi")
    #     self.assertFalse(searchBtn.is_enabled())
    #     refInput = self.driver.find_element(by=AppiumBy.NAME, value="Order number or booking reference")
    #     refInput.send_keys("1234567")
    #     self.assertFalse(searchBtn.is_enabled())
    #     refInput.send_keys("12345")
    #     self.assertTrue(searchBtn.is_enabled())
    #     refInput.clear()
    #     self.assertFalse(searchBtn.is_enabled())
    #     refInput.send_keys("ABC123")
    #     self.assertTrue(searchBtn.is_enabled())
    #
    #     self.driver.find_element(by=AppiumBy.NAME, value="Delete trip").click()
    #     self.driver.find_element(by=AppiumBy.NAME, value="Delete").click()

    def test_sncf(self):
        self.createAndEnterTrip("My New Trip")

        action = self.driver.find_element(by=AppiumBy.NAME, value="From SNCF reservation number")
        self.assertTrue(action.is_enabled())
        action.click()

        searchBtn = self.driver.find_element(by=AppiumBy.NAME, value="Search…")
        self.assertFalse(searchBtn.is_enabled())
        nameInput = self.driver.find_element(by=AppiumBy.NAME, value="Family name")
        nameInput.send_keys("Konqi")
        self.assertFalse(searchBtn.is_enabled())
        refInput = self.driver.find_element(by=AppiumBy.NAME, value="Booking reference")
        refInput.send_keys("ABC123")
        self.assertTrue(searchBtn.is_enabled())
        self.goBack()

        self.driver.find_element(by=AppiumBy.NAME, value="Delete trip").click()
        self.driver.find_element(by=AppiumBy.NAME, value="Delete").click()

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# SPDX-License-Identifier: LGPL-2.0-or-later
# SPDX-FileCopyrightText: 2023 Volker Krause <vkrause@kde.org>

from itinerarytestcase import ItineraryTestCase

import unittest
from appium.webdriver.common.appiumby import AppiumBy
from selenium.webdriver.common.keys import Keys


class SettingsTest(ItineraryTestCase):
    def test_settings(self):
        self.openGlobalDrawer()
        self.driver.find_element(by=AppiumBy.NAME, value="Settings…").click()

        # we run in en-US locale, so the metric unit switch has to exist
        metricSwitch = self.driver.find_element(by=AppiumBy.NAME, value="Use metric units")
        self.assertTrue(metricSwitch.is_displayed())
        metricSwitch.click()

        trafficData = self.driver.find_element(by=AppiumBy.NAME, value="Query Traffic Data")
        self.assertEqual(trafficData.get_attribute("checked"), "false")
        trafficData.send_keys(Keys.SPACE)  # TODO there is no toggle()?
        self.assertEqual(trafficData.get_attribute("checked"), "true")

        self.driver.find_element(by=AppiumBy.NAME, value="Public Transport Information Sources…").click()
        self.goBack()
        self.goBack()

        self.openGlobalDrawer()
        self.driver.find_element(by=AppiumBy.NAME, value="Settings…").click()
        self.assertEqual(self.driver.find_element(by=AppiumBy.NAME, value="Query Traffic Data").get_attribute("checked"), "true")
        self.goBack()


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# SPDX-License-Identifier: LGPL-2.0-or-later
# SPDX-FileCopyrightText: 2023 Volker Krause <vkrause@kde.org>

from itinerarytestcase import ItineraryTestCase

import unittest
from appium import webdriver
from appium.options.common.base import AppiumOptions
from appium.webdriver.common.appiumby import AppiumBy
import selenium.common.exceptions
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import WebDriverWait

import os

# Misc tests that aren't large enough (yet) to justify their own file
class ItineraryTest(ItineraryTestCase):
    def test_addTrainTrip(self):
        self.driver.find_element(by=AppiumBy.NAME, value="Plan Trip").click()

        self.driver.find_element(by=AppiumBy.NAME, value="From:").click()
        self.driver.find_element('description', value="Select country").click()
        self.driver.find_element(by=AppiumBy.NAME, value="Switzerland").click()
        searchField = self.driver.find_element(by=AppiumBy.NAME, value="Search")
        self.assertTrue(searchField.is_displayed())
        searchField.clear()
        searchField.send_keys("Randa")
        self.driver.find_element(by=AppiumBy.NAME, value="Randa, Valais, Switzerland").click()

        self.assertFalse(self.driver.find_element(by=AppiumBy.NAME, value="Search Journey").is_enabled())

        self.driver.find_element(by=AppiumBy.NAME, value="To:").click()
        # exists in history
        self.assertTrue(self.driver.find_element(by=AppiumBy.NAME, value="Randa, Valais, Switzerland").is_displayed())

        # clear history
        self.openContextDrawer()
        self.assertTrue(self.driver.find_element(by=AppiumBy.NAME, value="Clear history").is_enabled())
        self.driver.find_element(by=AppiumBy.NAME, value="Clear history").click()
        self.driver.find_element(by=AppiumBy.NAME, value="Cancel").click()
        self.assertTrue(self.driver.find_element(by=AppiumBy.NAME, value="Randa, Valais, Switzerland").is_displayed())
        self.openContextDrawer()
        self.driver.find_element(by=AppiumBy.NAME, value="Clear history").click()
        self.driver.find_element(by=AppiumBy.NAME, value="Remove").click()
        # self.assertFalse(self.driver.find_element(by=AppiumBy.NAME, value="Randa, Valais, Switzerland").is_displayed())
        self.goBack()


    def test_import(self):
        self.driver.find_element(by=AppiumBy.NAME, value="My Data").click()
        self.driver.find_element(by=AppiumBy.NAME, value="Manage your Passes").click()
        self.driver.find_element(by=AppiumBy.NAME, value="Import").click()
        self.driver.find_element(by=AppiumBy.NAME, value="From barcode").click()

        self.assertTrue(self.driver.find_element(by=AppiumBy.NAME, value="No camera available.").is_displayed())
        self.assertFalse(self.driver.find_element(by=AppiumBy.NAME, value="Light").is_enabled())
        self.goBack()

    def test_about(self):
        self.openGlobalDrawer()
        self.driver.find_element(by=AppiumBy.NAME, value="About").click()
        self.goBack()

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# SPDX-License-Identifier: LGPL-2.0-or-later
# SPDX-FileCopyrightText: 2023 Volker Krause <vkrause@kde.org>

from itinerarytestcase import ItineraryTestCase

import unittest
import time
from appium import webdriver
from appium.webdriver.common.appiumby import AppiumBy

import os
import time

class PassImportTest(ItineraryTestCase):
    def setUp(self):
        # open the passes & programs page
        self.driver.find_element(by=AppiumBy.NAME, value="My Data").click()
        self.driver.find_element(by=AppiumBy.NAME, value="Manage your Passes").click()

    def test_programMembership(self):
        self.assertTrue(self.driver.find_element(by=AppiumBy.NAME, value="Import From File").is_displayed())
        self.assertTrue(self.driver.find_element(by=AppiumBy.NAME, value="Import From File").is_enabled())
        self.driver.find_element(by=AppiumBy.NAME, value="Import From File").click()
        self.openFile("data/bahncard.json")
        self.driver.find_element(by=AppiumBy.NAME, value="Import selection").click()
        self.driver.find_element(by=AppiumBy.NAME, value="BahnCard 25 (2. Kl.) (BC25)").click()
        self.driver.find_element(by=AppiumBy.NAME, value="Edit").click()

        saveButton = self.driver.find_element(by=AppiumBy.NAME, value="Save")
        self.assertTrue(saveButton.is_enabled())

        nameInput = self.driver.find_element(by=AppiumBy.NAME, value="Name")
        nameInput.clear()
        self.assertFalse(saveButton.is_enabled())
        nameInput.send_keys("BahnCard 50")
        self.assertTrue(saveButton.is_enabled())
        saveButton.click();

        self.goBack()
        self.assertTrue(self.driver.find_element(by=AppiumBy.NAME, value="BahnCard 50").is_displayed())

        self.driver.find_element(by=AppiumBy.NAME, value="BahnCard 50").click()
        self.driver.find_element(by=AppiumBy.NAME, value="Delete").click()

        deleteDialog = self.driver.find_element(by=AppiumBy.NAME, value="Delete Pass")
        self.assertTrue(deleteDialog.is_displayed())
        deleteDialog.find_element(by=AppiumBy.NAME, value="Delete").click()
        # TODO how to check this element does not exist anymore?
        # self.assertFalse(self.driver.find_element(by=AppiumBy.NAME, value="BahnCard 25 (2. Kl.) (BC25)").is_displayed())

class PassManualTest(ItineraryTestCase):
    def setUp(self):
        # open the passes & programs page
        self.driver.find_element(by=AppiumBy.NAME, value="My Data").click()
        self.driver.find_element(by=AppiumBy.NAME, value="Manage your Passes").click()

    def test_addProgramMembership(self):
        self.driver.find_element(by=AppiumBy.NAME, value="Add Manually").click()
        saveButton = self.driver.find_element(by=AppiumBy.NAME, value="Save")
        self.assertFalse(saveButton.is_enabled())

        self.driver.find_element(by=AppiumBy.NAME, value="Name").send_keys("New Discount Card")
        self.assertTrue(saveButton.is_enabled())
        self.driver.find_element(by=AppiumBy.NAME, value="Member").send_keys("Konqi Dragon")
        self.driver.find_element(by=AppiumBy.NAME, value="Number").send_keys("1234567890")

        validFrom = self.driver.find_elements(by=AppiumBy.NAME, value="Valid from")[0]
        validFrom.find_elements(by=AppiumBy.NAME, value="Not set")[1].click()
        hours = self.driver.find_element(by=AppiumBy.NAME, value="Hours")
        self.assertEqual(float(hours.get_attribute('value')), 0)
        self.driver.set_value(hours, int(6))
        minutes = self.driver.find_element(by=AppiumBy.NAME, value="Minutes")
        self.assertEqual(float(minutes.get_attribute('value')), 0)
        self.driver.set_value(minutes, int(30))
        self.driver.find_element(by=AppiumBy.NAME, value="Select").click()

        validFrom.find_element(by=AppiumBy.NAME, value="Today").click()
        days = self.driver.find_element(by=AppiumBy.NAME, value="Day")
        self.assertTrue(float(days.get_attribute('value')) >= 1 and float(days.get_attribute('value')) <= 31)
        self.driver.set_value(days, int(7))
        months = self.driver.find_element(by=AppiumBy.NAME, value="Month")
        self.assertTrue(float(months.get_attribute('value')) >= 1 and float(months.get_attribute('value')) <= 12)
        self.driver.set_value(months, int(9))
        years = self.driver.find_element(by=AppiumBy.NAME, value="Year")
        self.assertTrue(float(years.get_attribute('value')) >= 2024)
        self.driver.set_value(years, int(2023))
        self.driver.find_element(by=AppiumBy.NAME, value="Select").click()

        self.assertTrue(saveButton.is_enabled())
        saveButton.click()

        self.driver.find_element(by=AppiumBy.NAME, value="New Discount Card").click()
        page = self.driver.find_element(by=AppiumBy.NAME, value="Program Membership")
        self.assertTrue(page.find_element(by=AppiumBy.NAME, value="New Discount Card").is_displayed())
        self.assertTrue(page.find_element(by=AppiumBy.NAME, value="1234567890").is_displayed())
        self.assertTrue(page.find_element(by=AppiumBy.NAME, value="Konqi Dragon").is_displayed())

        self.driver.find_element(by=AppiumBy.NAME, value="Delete").click()
        deleteDialog = self.driver.find_element(by=AppiumBy.NAME, value="Delete Pass")
        self.assertTrue(deleteDialog.is_displayed())
        deleteDialog.find_element(by=AppiumBy.NAME, value="Delete").click()

if __name__ == '__main__':
    unittest.main()
